package profiler;

import javax.swing.table.AbstractTableModel;
import shared.GUV;
import shared.GUVSet;

class GUVTableModel extends AbstractTableModel {

   private static final String[] HEADERS = {"#", "Used", "Mean", "+/- SD"};

   GUVSet set;

   private final Profiler profiler;

   private int channel;

   private boolean isTransmittedChannel;

   public GUVTableModel(Profiler profiler) {
      this.profiler = profiler;
      channel = 0;
   }

   @Override
   public int findColumn(String name) {
      for (int i = 0; i < HEADERS.length; i++) {
         if (HEADERS[i].equals(name)) {
            return i;
         }
      }
      return 0;
   }

   @Override
   public int getColumnCount() {
      return HEADERS.length;
   }

   @Override
   public Class<?> getColumnClass(int col) {
      switch (col) {
         case 0: return Integer.class;
         case 1: return Boolean.class;
         case 2:
         case 3:
            return Double.class;
         default: return String.class;
      }
   }

   @Override
   public String getColumnName(int col) {
      return HEADERS[col];
   }

   @Override
   public int getRowCount() {
      if (set == null || isTransmittedChannel) {
         return 0;
      }
      return set.count();
   }

   @Override
   public Object getValueAt(int row, int col) {
      GUV guv = set.get(row);
      if (guv == null) {
         return null;
      }
      switch (col) {
         case 0: return guv.id;
         case 1: return guv.isUsed();
         case 2: return guv.mean[channel];
         case 3: return guv.sd[channel]; //standard deviation
         default: return null;
      }
   }

   @Override
   public boolean isCellEditable(int row, int col) {
      return col == 1;
   }

   @Override
   public void setValueAt(Object value, int row, int col) {
      if (col == 1) {
         GUV guv = set.get(row);
         guv.setUsed((boolean) value);
         set.calculate(channel);
         fireTableCellUpdated(row, col);
         profiler.updateStatistics(channel);
      }
   }

   GUV get(int index) {
      return set.get(index);
   }

   void set(GUVSet set) {
      this.set = set;
   }

   /**Sets the active channel.
    * @param channel the channel, starting with 0
    * @param isTransmittedChannel <code>true</code> indicates a transmitted light channel (not displayed)*/
   void setChannel(int channel, boolean isTransmittedChannel) {
      this.channel = channel;
      this.isTransmittedChannel = isTransmittedChannel;
      fireTableDataChanged();
   }

   /**Clears the content of this model.*/
   void reset() {
      set = null;
      isTransmittedChannel = false;
      fireTableDataChanged();
   }

   void update(int index) {
      set.get(index).calculate(channel);
      set.calculate(channel);
      fireTableRowsUpdated(index, index);
      profiler.updateStatistics(channel);
   }
}
