package profiler;

import java.util.ArrayList;
import java.util.List;
import javax.swing.table.AbstractTableModel;
import shared.GUV;
import shared.Profile;

class ProfileTableModel extends AbstractTableModel {

   private static final String[] HEADERS = {"#", "Used", "Intensity", "Radius"};
   private final List<Profile> emptyList;
   private final GUVTableModel guvTableModel;

   private List<Profile> profiles;
   private int guvIndex;
   private int channel;

   ProfileTableModel(GUVTableModel guvTableModel) {
      this.guvTableModel = guvTableModel;
      emptyList = new ArrayList<>(0);
      profiles = emptyList;
   }

   @Override
   public int findColumn(String name) {
      for (int i = 0; i < HEADERS.length; i++) {
         if (HEADERS[i].equals(name)) {
            return i;
         }
      }
      return 0;
   }

   @Override
   public Class<?> getColumnClass(int col) {
      switch (col) {
         case 0: return Integer.class;
         case 1: return Boolean.class;
         case 2:
         case 3:
            return Double.class;
         default: return String.class;
      }
   }

   @Override
   public int getColumnCount() {
      return HEADERS.length;
   }

   @Override
   public String getColumnName(int col) {
      return HEADERS[col];
   }

   @Override
   public int getRowCount() {
      return profiles.size();
   }

   @Override
   public Object getValueAt(int row, int col) {
      Profile p = profiles.get(row);
      switch (col) {
         case 0: return p.id;
         case 1: return p.isUsed();
         case 2: return p.intensity[channel];
         case 3: return p.radius[channel];
         default: return null;
      }
   }

   @Override
   public boolean isCellEditable(int row, int col) {
      return col == 1;
   }

   @Override
   public void setValueAt(Object value, int row, int col) {
      if (col == 1) {
         GUV guv = guvTableModel.get(guvIndex);
         Profile p = guv.profiles.get(row);
         p.setUsed((boolean) value);
         fireTableCellUpdated(row, col);
         guvTableModel.update(guvIndex);
      }
   }

   /**Sets the active channel.
    * @param channel the channel, starting with 0*/
   void setChannel(int channel) {
      this.channel = channel;
      fireTableDataChanged();
   }

   /**Sets the current GUV index.
    * @param index the index*/
   void setGUV(int index) {
      guvIndex = index;
      if (index == -1) {
         profiles = emptyList;
      } else {
         profiles = guvTableModel.get(index).profiles;
      }
      fireTableDataChanged();
   }
}
