package segmentation;

import javax.swing.SwingWorker;
import shared.IJLogger;

/**Base class for processors*/
abstract class AbstractProcessor extends SwingWorker<Void, Void> {

   protected final IJLogger logger;
   protected final BatchInfo batchInfo;

   private double increment;
   private double progress;

   AbstractProcessor(IJLogger logger, BatchInfo batchInfo) {
      this.logger = logger;
      this.batchInfo = batchInfo;
      progress = 0;
      addPropertyChangeListener(batchInfo.listener);
   }

   @Override
   protected abstract Void doInBackground() throws Exception;

   /**Subclasses should announce the progress increment here.*/
   protected void setIncrement(double increment) {
      this.increment = increment;
   }

   protected void incrementProgress() {
      progress += increment;
      setProgress((int) Math.floor(progress));
   }
}
