package shared;

import ij.ImagePlus;
import ij.gui.Line;
import ij.gui.Overlay;
import ij.gui.Plot;
import ij.gui.PointRoi;
import java.awt.geom.Point2D;
import java.util.Arrays;
import java.util.stream.IntStream;

public class Profile {

   private static final int STROKE = 1;
   public int id;
   public Line line;
   public double[] intensity, radius;
   int fitStart = 0; //start of the fitted region (at about the half maximum inside the vesicle)
   private boolean use = true;

   Profile(int profileID, Line line, int nChannels) {
      this.id = profileID;
      this.line = line;
      line.setStrokeWidth(STROKE);
      radius = new double[nChannels];
      intensity = new double[nChannels];
   }

   @SuppressWarnings("unused")
   static void plot(Profile p) {
      double[] values = p.line.getPixels();
      int[] ints = IntStream.range(p.fitStart, values.length).toArray();
      double[] valuesX = Arrays.stream(ints).asDoubleStream().toArray();
      double[] valuesY = Arrays.copyOfRange(p.line.getPixels(), p.fitStart, values.length);
      Plot plot = new Plot("Line "+p.id, "Distance (pixel)", "Intensity");
      plot.add("line", valuesX, valuesY);
      plot.show();
   }

   public void adjustLine(ImagePlus imp, PointRoi roi) {
      Point2D p = roi.getContainedPoints()[0];
      Line line = new Line(this.line.x1d, this.line.y1d, p.getX(), p.getY());
      line.setStrokeWidth(STROKE);
      Overlay overlay = imp.getOverlay();
      overlay.remove(this.line);
      this.line = line;
      overlay.add(this.line, String.valueOf(id));
      measureAll(imp);
   }

   public boolean isUsed() {
      return use;
   }

   /**Measures the intensity of the current position of the image provided*/
   public void measure(ImagePlus imp) {
      int channel = imp.getChannel() - 1;
      imp.setRoi(line);
      double min = Double.MAX_VALUE;
      double max = 0;
      double[] values = line.getPixels();
      for (int i = 0; i < values.length; i++) {
         double value = values[i];
         if (value > max) {
            max = value;
            radius[channel] = i;
         } else if (value < min) {
            min = value;
         }
      }
      intensity[channel] = max - min;
      line.setProperty("intensity-ch"+channel, String.valueOf(intensity[channel]));
      line.setProperty("radius-ch"+channel, String.valueOf(radius[channel]));
      imp.deleteRoi();
   }

   /**Measures the intensity of all channels of the current slice and frame of the image provided.
    * @param imp the image*/
   public void measureAll(ImagePlus imp) {
      int currentChannel = imp.getC();
      for (int i = 1; i <= imp.getNChannels(); i++) {
         imp.setC(i);
         measure(imp);
      }
      imp.setC(currentChannel);
   }

   public void setUsed(boolean use) {
      this.use = use;
      line.setProperty("use", String.valueOf(use));
   }
}
